@@ -27,6 +27,17 @@
 #include "statusbar-item.h"
 #include <gtk/gtkwindow.h>
 #include <gtk/gtktable.h>
+#define SYSTRAY_SUPPORT 1
+#ifdef SYSTRAY_SUPPORT
+#include <X11/Xlib.h>
+#include <gdk/gdkx.h>
+#include <gdk/gdkevents.h>
+#include <libhildondesktop/statusbar-item-socket.h>
+#include <gtk/gtkinvisible.h>
+#define SYSTEM_TRAY_REQUEST_DOCK    0
+#define SYSTEM_TRAY_BEGIN_MESSAGE   1
+#define SYSTEM_TRAY_CANCEL_MESSAGE  2
+#endif
 
 #define HILDON_DESKTOP_PANEL_EXPANDABLE_GET_PRIVATE(object) \
         (G_TYPE_INSTANCE_GET_PRIVATE ((object), HILDON_DESKTOP_TYPE_PANEL_EXPANDABLE, HildonDesktopPanelExpandablePrivate))
@@ -41,6 +52,7 @@ G_DEFINE_TYPE (HildonDesktopPanelExpandable, hildon_desktop_panel_expandable, HI
 enum 
 {
   SIGNAL_QUEUED_BUTTON,
+  SIGNAL_ARROW_ADDED,
   N_SIGNALS
 };
 
@@ -62,6 +74,11 @@ struct _HildonDesktopPanelExpandablePrivate
   gboolean    extension_opened;
 
   GtkWidget  *arrow;
+#ifdef SYSTRAY_SUPPORT
+  Atom 	      tray_opcode;
+  GtkWidget  *invisible;
+  gboolean    filter_added;
+#endif
 };
 
 enum
@@ -104,6 +121,12 @@ static void hildon_desktop_panel_expandable_arrow_toggled (GtkToggleButton *butt
 
 static void hildon_desktop_panel_expandable_resize_notify (HildonDesktopPanelExpandable *panel);
 
+#ifdef SYSTRAY_SUPPORT
+static void hildon_desktop_panel_expandable_init_systray (HildonDesktopPanelExpandable *panel, gpointer data);
+static GdkFilterReturn hildon_desktop_x_event_filter (GdkXEvent *xevent, GdkEvent *event, gpointer _panel);
+static void hildon_desktop_panel_embed_applet (HildonDesktopPanelExpandable *panel, Window wid);
+#endif
+
 static void 
 hildon_desktop_panel_expandable_class_init (HildonDesktopPanelExpandableClass *panel_class)
 {
@@ -142,6 +165,14 @@ hildon_desktop_panel_expandable_class_init (HildonDesktopPanelExpandableClass *p
                      g_cclosure_marshal_VOID__OBJECT,
                      G_TYPE_NONE, 1, GTK_TYPE_WIDGET);
 
+  signals[SIGNAL_ARROW_ADDED] =
+        g_signal_new("arrow-added",
+                     G_OBJECT_CLASS_TYPE(object_class),
+                     G_SIGNAL_RUN_FIRST,
+		     0, NULL, NULL,
+                     g_cclosure_marshal_VOID__OBJECT,
+                     G_TYPE_NONE, 1, GTK_TYPE_WIDGET);
+
 }
 
 static void 
@@ -159,11 +190,11 @@ hildon_desktop_panel_expandable_init (HildonDesktopPanelExpandable *panel)
 					      (GDestroyNotify) gtk_widget_destroy);
 
   panel->priv->queued_items = NULL;
-  
   panel->priv->extension_opened = FALSE;
-
   panel->priv->extension_table = NULL;
-
+#ifdef SYSTRAY_SUPPORT
+  panel->priv->filter_added = FALSE;
+#endif
 }
 
 GObject *
@@ -207,7 +238,13 @@ hildon_desktop_panel_expandable_constructor (GType gtype,
     g_object_set (object, "item_width", HSB_ITEM_WIDTH, "item_height", HSB_ITEM_HEIGHT, NULL);
 
   HILDON_DESKTOP_PANEL (panel)->pack_start = FALSE;
-  
+
+#ifdef SYSTRAY_SUPPORT
+  g_signal_connect_after (panel,
+		          "realize",
+		          G_CALLBACK (hildon_desktop_panel_expandable_init_systray),
+		          NULL);
+#endif
   return object;
 }
 
@@ -219,7 +256,15 @@ hildon_desktop_panel_expandable_finalize (GObject *object)
   gtk_widget_destroy (GTK_WIDGET (panel->priv->extension_window));
 
   g_hash_table_destroy (panel->priv->items);
+#ifdef SYSTRAY_SUPPORT
+  if (panel->priv->invisible)
+    gtk_widget_destroy (panel->priv->invisible);
 
+  if (panel->priv->filter_added)
+    gdk_window_remove_filter (panel->priv->invisible->window,
+		  	      hildon_desktop_x_event_filter,
+			      panel);
+#endif
   G_OBJECT_CLASS (hildon_desktop_panel_expandable_parent_class)->finalize (object);
 }
 
@@ -298,13 +343,19 @@ hildon_desktop_panel_expandable_add_button (HildonDesktopPanel *panel, GtkWidget
   HildonDesktopPanelItem *item = HILDON_DESKTOP_PANEL_ITEM (button);
   HildonDesktopPanelExpandable *ex_panel = HILDON_DESKTOP_PANEL_EXPANDABLE (panel);
   gint item_width,item_height;
+  GtkRequisition req;
 
   g_signal_emit_by_name (ex_panel, "queued-button", button);
 
   g_debug ("Adding button in expandable %d %d",ex_panel->priv->items_p_row,ex_panel->priv->n_items+1);
 
-  g_object_get (G_OBJECT (panel), "item_width", &item_width, "item_height", &item_height, NULL);
+  gtk_widget_size_request (button, &req);
   
+  g_object_get (G_OBJECT (panel), "item_width", &item_width, "item_height", &item_height, NULL);
+
+  if (req.width > item_width)
+    item_width = req.width;
+	  
   gtk_widget_set_size_request (button, item_width, item_height);
 
   g_signal_connect (button,
@@ -468,14 +519,19 @@ hildon_desktop_panel_expandable_arrange_items (HildonDesktopPanelExpandable *pan
       g_object_ref (G_OBJECT (l->data));
     else
       panel->priv->arrow = NULL;
-    gtk_container_remove (GTK_CONTAINER (panel), GTK_WIDGET (l->data));
+
+    if (!STATUSBAR_IS_ITEM_SOCKET (l->data))
+      gtk_container_remove (GTK_CONTAINER (panel), GTK_WIDGET (l->data));
   }
 
   for (l = children_table ; l ; l = g_list_next (l))
   {
-    g_object_ref (G_OBJECT (l->data));
-    gtk_container_remove (GTK_CONTAINER (panel->priv->extension_table),
-		    	  GTK_WIDGET (l->data));
+    if (!STATUSBAR_IS_ITEM_SOCKET (l->data))
+    {	    
+      g_object_ref (G_OBJECT (l->data));
+      gtk_container_remove (GTK_CONTAINER (panel->priv->extension_table),
+		    	    GTK_WIDGET (l->data));
+    }
   }
 
   panel->priv->n_items = panel->priv->current_position = 0;
@@ -495,6 +551,8 @@ hildon_desktop_panel_expandable_arrange_items (HildonDesktopPanelExpandable *pan
 
     if (STATUSBAR_IS_ITEM (l->data) && !STATUSBAR_ITEM (l->data)->condition)
       g_debug ("not unreffing");
+    else if (STATUSBAR_IS_ITEM (l->data) && STATUSBAR_ITEM (l->data)->mandatory)
+      g_debug ("not unreffing");
     else
       g_object_unref (G_OBJECT (l->data));
   }	  
@@ -548,7 +606,9 @@ hildon_desktop_panel_expandable_add_arrow (HildonDesktopPanelExpandable *panel)
   gtk_widget_set_size_request (arrow_image, item_width, item_height);
 
   gtk_widget_show (arrow_button);
-  
+
+  g_signal_emit_by_name (panel, "arrow-added", arrow_button);
+		  
   return arrow_button;
 }
 
@@ -610,3 +670,165 @@ hildon_desktop_panel_expandable_resize_notify (HildonDesktopPanelExpandable *pan
 
   hildon_desktop_panel_expandable_arrange_items (panel);  
 }
+
+#ifdef SYSTRAY_SUPPORT
+static void 
+hildon_desktop_panel_expandable_init_systray (HildonDesktopPanelExpandable *panel, gpointer data)
+{
+  Display *display;	
+  Window root;
+  Atom tray_selection;
+  gchar *tray_string;
+
+  display = GDK_DISPLAY ();
+
+  tray_string = 
+    g_strdup_printf ("_NET_SYSTEM_TRAY_S%d", 
+		     GDK_SCREEN_XNUMBER (gdk_screen_get_default ()));
+  
+  tray_selection = XInternAtom (display, tray_string, False);
+
+  panel->priv->tray_opcode = XInternAtom (display, "_NET_SYSTEM_TRAY_OPCODE", False);
+
+  panel->priv->invisible = gtk_invisible_new_for_screen (gdk_screen_get_default ());
+  
+  gtk_widget_realize (panel->priv->invisible);
+  gtk_widget_add_events (panel->priv->invisible,
+		  	 GDK_PROPERTY_CHANGE_MASK | GDK_STRUCTURE_MASK);
+
+  XSetSelectionOwner (display,
+                      tray_selection,
+		      GDK_WINDOW_XID (panel->priv->invisible->window),
+                      CurrentTime);
+
+  root = GDK_WINDOW_XID (gdk_get_default_root_window ());
+
+  if (XGetSelectionOwner (display, tray_selection) == GDK_WINDOW_XID (panel->priv->invisible->window))
+  {
+    XClientMessageEvent xev;
+
+    xev.type = ClientMessage;
+    xev.window = root;
+    xev.message_type = XInternAtom (display, "MANAGER", False);
+    xev.format = 32;
+    xev.data.l[0] = CurrentTime;
+    xev.data.l[1] = tray_selection;
+    xev.data.l[2] = GDK_WINDOW_XID (panel->priv->invisible->window);
+    xev.data.l[3] = 0;
+    xev.data.l[4] = 0;
+
+    gdk_error_trap_push ();
+
+    XSendEvent (display, root, False, StructureNotifyMask, (XEvent *)&xev);
+
+    gdk_error_trap_pop ();
+
+    gdk_window_add_filter (panel->priv->invisible->window,
+			   hildon_desktop_x_event_filter,
+			   (gpointer)panel);
+
+    panel->priv->filter_added = TRUE;
+  }
+
+  g_free (tray_string);
+}
+
+static GdkFilterReturn 
+hildon_desktop_x_event_filter (GdkXEvent *xevent,
+			       GdkEvent *event,
+			       gpointer _panel)
+{
+  HildonDesktopPanelExpandable *panel = 
+    HILDON_DESKTOP_PANEL_EXPANDABLE (_panel);
+
+  XEvent *e = (XEvent*)xevent;
+
+  if (e->type == ClientMessage)
+  {
+    if (e->xclient.message_type == panel->priv->tray_opcode &&
+        e->xclient.data.l[1] == SYSTEM_TRAY_REQUEST_DOCK)
+    {
+      gchar *id = g_strdup_printf ("XEmbed:%d",(GdkNativeWindow)e->xclient.data.l[2]);
+	    
+      if (g_hash_table_lookup (panel->priv->items, id) != NULL)
+      {
+	g_free (id);
+        return GDK_FILTER_CONTINUE;
+      }
+
+      g_free (id);
+      
+      hildon_desktop_panel_embed_applet (panel,(Window)e->xclient.data.l[2]);
+      return GDK_FILTER_CONTINUE;
+    }
+  }
+
+  return TRUE;
+}
+
+static gboolean 
+hildon_desktop_panel_remove_embed (GtkSocket *socket, HildonDesktopPanelExpandable *panel)
+{
+  GtkWidget *parent;
+  gchar *id = NULL;
+	
+  parent = gtk_widget_get_parent (GTK_WIDGET (socket));
+
+  id = g_strdup (HILDON_DESKTOP_ITEM (parent)->id);
+
+  g_debug ("%s ---- has been removed",id);
+  
+  g_hash_table_steal (panel->priv->items, id);
+
+  panel->priv->n_items--;
+
+  g_free (id);
+
+  return FALSE;
+}
+
+static void 
+hildon_desktop_panel_embed_applet (HildonDesktopPanelExpandable *panel, Window wid)
+{
+  StatusbarItemSocket *sb_socket;
+  gchar *id, *name;
+
+  id   = g_strdup_printf ("XEmbed:%d",(GdkNativeWindow)wid);
+  name = g_strdup_printf ("Statusbar-systray-item:%d",(GdkNativeWindow)wid);
+
+  g_debug ("id: %s ------------ name: %s",id,name);
+  
+  sb_socket = g_object_new (STATUSBAR_TYPE_ITEM_SOCKET,
+		  	    "id",id,
+			    "name",name,
+			    "position",0,
+			    "mandatory",TRUE,
+			    NULL);
+
+  g_signal_connect (sb_socket->socket,
+		    "plug-removed",
+		    G_CALLBACK (hildon_desktop_panel_remove_embed),
+		    panel);
+
+  hildon_desktop_panel_expandable_add_button (HILDON_DESKTOP_PANEL (panel),
+		  			      GTK_WIDGET (sb_socket));
+  
+  hildon_desktop_panel_expandable_arrange_items (panel);
+
+  hildon_desktop_item_socket_add_id (HILDON_DESKTOP_ITEM_SOCKET (sb_socket), 
+		  		     wid);
+
+
+  g_free (id);
+  g_free (name);
+}
+#endif
+
+GtkWidget *
+hildon_desktop_panel_expandable_get_extension (HildonDesktopPanelExpandable *panel)
+{
+  g_assert (HILDON_DESKTOP_IS_PANEL_EXPANDABLE (panel));
+
+  return GTK_WIDGET (panel->priv->extension_table);
+}
+
