@@ -35,10 +35,11 @@
 #include <gtk/gtkimage.h>
 #include <gtk/gtkmain.h>
 
-#include <hildon-widgets/hildon-banner.h>
-#include <hildon-widgets/hildon-note.h>
+#ifdef HAVE_LIBHILDON
+#include <hildon/hildon-helper.h>
+#else
 #include <hildon-widgets/hildon-defines.h>
-#include <hildon-widgets/hildon-caption.h>
+#endif
 
 #include "hildon-home-titlebar.h"
 #include "hildon-home-area.h"
@@ -460,14 +461,23 @@ hildon_home_titlebar_init (HildonHomeTitlebar *titlebar)
   priv->label = gtk_label_new (priv->normal_text);
   gtk_widget_set_composite_name (priv->label, "hildon-home-titlebar-label");
   gtk_misc_set_alignment (GTK_MISC (priv->label), 0.0, 0.5);
+
+#ifdef HAVE_LIBHILDON
+  hildon_helper_set_logical_font (priv->label,
+                                  HILDON_HOME_TITLEBAR_MENU_LABEL_FONT);
+  hildon_helper_set_logical_color (priv->label,
+                                   GTK_RC_FG,
+                                   GTK_STATE_NORMAL,
+                                   HILDON_HOME_TITLEBAR_MENU_LABEL_COLOR);
+#else
   hildon_gtk_widget_set_logical_font (priv->label,
                                       HILDON_HOME_TITLEBAR_MENU_LABEL_FONT);
-  /*
   hildon_gtk_widget_set_logical_color (priv->label,
                                        GTK_RC_FG,
                                        GTK_STATE_NORMAL,
                                        HILDON_HOME_TITLEBAR_MENU_LABEL_COLOR);
-  */
+#endif
+
   gtk_container_add (GTK_CONTAINER (align), priv->label);
   gtk_widget_show (priv->label);
 
