@@ -51,7 +51,9 @@
 
 #include <hildon-base-lib/hildon-base-dnotify.h>
 
+#ifdef HAVE_LIBOSSO
 #include <libosso.h>
+#endif
 
 #define DBUS_API_SUBJECT_TO_CHANGE
 #include <dbus/dbus.h>
@@ -114,6 +116,7 @@ enum
   HDWM_WORK_AREA_CHANGED_SIGNAL,
   HDWM_SHOW_A_MENU_SIGNAL,
   HDWM_APPLICATION_STARTING_SIGNAL,
+  HDWM_FULLSCREEN,
   HDWM_SIGNALS
 };
 
@@ -147,6 +150,9 @@ hd_wm_process_x_client_list (HDWM *hdwm);
 static gboolean
 hd_wm_relaunch_timeout (gpointer data);
 
+	static void 
+hd_wm_check_net_state (HDWM *hdwm, HDWMWatchedWindow *win);
+
 struct xwinv
 {
   Window *wins;
@@ -594,53 +600,6 @@ mce_handler (DBusConnection *conn,
   return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
 }
 
-static int
-hd_wm_osso_kill_method (GArray *arguments, gpointer data)
-{
-  gchar             *appname = NULL, *operation = NULL;
-  HDWMWatchedWindow *win = NULL;
-  HDWMWatchableApp  *app;
-
-  if (arguments->len < 1)
-    return 1;
-
-  operation = (gchar *)g_array_index(arguments, osso_rpc_t, 0).value.s;
-
-  /* In this case, HN will kill every process that has supposedly
-   * statesaved
-   */
-
-  if (operation == NULL)
-    return 1;
-
-  if (strcmp(operation, "lru") == 0)
-    return hd_wm_memory_kill_lru ();
-  else 
-  if (strcmp(operation, "all") == 0)
-    return hd_wm_memory_kill_all_watched (TRUE);
-  else
-  if (strcmp(operation, "app") != 0 || (arguments->len < 2) )
-    return 1;
-  
-  /* Kill a certain application */
-
-  appname = (gchar *)g_array_index(arguments, osso_rpc_t, 1).value.s;
-
-  if (appname == NULL)
-    return 1;
-
-  win = hd_wm_lookup_watched_window_via_service (appname);
-  app = hd_wm_watched_window_get_app (win);
-
-  if (win)
-    {
-      if (hd_wm_watchable_app_is_able_to_hibernate (app))
-        hd_wm_watched_window_attempt_signal_kill (win, SIGTERM, TRUE);
-    }
-
-  return 0;
-}
-
 static void 
 hd_wm_class_init (HDWMClass *hdwm_class)
 {
@@ -715,6 +674,17 @@ hd_wm_class_init (HDWMClass *hdwm_class)
                      1,
                      G_TYPE_STRING);
 
+  hdwm_signals[HDWM_FULLSCREEN] = 
+	g_signal_new("fullscreen",
+		     G_OBJECT_CLASS_TYPE(object_class),
+		     G_SIGNAL_RUN_LAST,
+		     G_STRUCT_OFFSET (HDWMClass,show_menu),
+		     NULL, NULL,
+		     g_cclosure_marshal_VOID__BOOLEAN,
+		     G_TYPE_NONE,
+		     1,
+		     G_TYPE_BOOLEAN);
+
 }
 
 static void hd_wm_register_object_path (HDWM *hdwm,
@@ -757,8 +727,6 @@ hd_wm_init (HDWM *hdwm)
   hdwm->priv = hdwmpriv = HD_WM_GET_PRIVATE (hdwm);
 	
   memset(hdwm->priv, 0, sizeof(HDWMPrivate));
-  
-  osso_manager_t *osso_man = osso_manager_singleton_get_instance();
 
   /* Check for configurable lowmem values. */
 
@@ -876,9 +844,6 @@ hd_wm_init (HDWM *hdwm)
 
     dbus_connection_flush(connection);
 
-    /* Add the com.nokia.tasknav callbacks - FIXME: what are these for ? */
-
-    add_method_cb (osso_man, KILL_APPS_METHOD, &hd_wm_osso_kill_method, osso_man);
   }
 
   if (!sys_connection)
@@ -1022,11 +987,62 @@ hd_wm_top_item (HDEntryInfo *info)
     g_debug  ("### Invalid window type ###\n");
 }
 
+void
+hd_wm_activate_service (const gchar *app, const gchar *parameters)
+{
+  char service[SERVICE_NAME_LEN],
+       path[PATH_NAME_LEN],
+       interface[INTERFACE_NAME_LEN],
+       tmp[TMP_NAME_LEN];
+    DBusMessage *msg = NULL;
+    DBusError error;
+    DBusConnection *conn;
+
+    /* If we have full service name we will use it*/
+    if (g_strrstr(app,"."))
+    {
+        g_snprintf(service,SERVICE_NAME_LEN,"%s",app);
+        g_snprintf(interface,INTERFACE_NAME_LEN,"%s",service);
+        g_snprintf(tmp,TMP_NAME_LEN,"%s",app);
+        g_snprintf(path,PATH_NAME_LEN,"/%s",g_strdelimit(tmp,".",'/'));
+    }
+    else /* we will use com.nokia prefix*/
+    {
+        g_snprintf(service,SERVICE_NAME_LEN,"%s.%s",OSSO_BUS_ROOT,app);
+        g_snprintf(path,PATH_NAME_LEN,"%s/%s",OSSO_BUS_ROOT_PATH,app);
+        g_snprintf(interface,INTERFACE_NAME_LEN,"%s",service);
+    }
+
+    conn = dbus_bus_get (DBUS_BUS_SESSION, &error);
+
+    if (dbus_error_is_set (&error))
+      {
+        g_warning ("could not start: %s: %s",
+                   service,
+                   error.message);
+        dbus_error_free (&error);
+        return;
+      }
+
+    msg = dbus_message_new_method_call (service,
+                                        path,
+                                        interface,
+                                        OSSO_BUS_TOP);
+    if (parameters)
+      dbus_message_append_args (msg,
+                                DBUS_TYPE_STRING,
+                                parameters,
+                                DBUS_TYPE_INVALID);
+
+    dbus_message_set_auto_start (msg, TRUE);
+    dbus_connection_send (conn, msg, NULL);
+}
+
+
 gboolean
 hd_wm_top_service (const gchar *service_name)
 {
   HDWM *hdwm = hd_wm_get_singleton ();
-  osso_manager_t    *osso_man;
   HDWMWatchedWindow *win;
   guint              pages_used = 0, pages_available = 0;
 
@@ -1090,9 +1106,8 @@ hd_wm_top_service (const gchar *service_name)
        * so just launch it.
        */
     g_debug ("unable to find service name '%s' in running wins", service_name);
-    g_debug ("Thus launcing via osso_manager_launch()");
-    osso_man = osso_manager_singleton_get_instance();
-    osso_manager_launch(osso_man, service_name, NULL);
+    g_debug ("Thus launcing via DBus");
+    hd_wm_activate_service(service_name, NULL);
     return TRUE;
   }
   else
@@ -1130,8 +1145,7 @@ hd_wm_top_service (const gchar *service_name)
         {
           /* we do not know which was the active window, so just launch it */
           hd_wm_watchable_app_set_launching (app, TRUE);
-          osso_man = osso_manager_singleton_get_instance();
-          osso_manager_launch (osso_man, hd_wm_watchable_app_get_service (app),RESTORED);
+          hd_wm_activate_service(service_name, RESTORED);
         }
 
       /*
@@ -1215,6 +1229,8 @@ hd_wm_toggle_desktop (void)
       const gchar * service = hd_wm_watchable_app_get_service (app);
       
       hd_wm_top_service (service);
+
+      hd_wm_check_net_state (hdwm,hdwm->priv->last_active_window);
     }
     else
       hd_wm_top_desktop ();
@@ -1252,6 +1268,8 @@ hd_wm_top_desktop(void)
    * this breaks the AS menu focus.
    */
   hdwm->priv->active_window = NULL;
+
+  g_signal_emit_by_name (hdwm, "fullscreen", FALSE);
 }
 
 
@@ -2097,6 +2115,67 @@ hdwm_power_key_timeout (gpointer data)
   return FALSE;  
 }
 
+static void 
+hd_wm_check_net_state (HDWM *hdwm, HDWMWatchedWindow *win)
+{
+  Window xid;
+  unsigned long n;
+  unsigned long extra;
+  int           format, status, i;
+  Atom          realType;
+  union
+  {
+     Atom *a;
+     unsigned char *cpp;
+  } value;
+
+  value.a = NULL;
+
+  if (hdwm == NULL)
+    hdwm = hd_wm_get_singleton ();
+
+  if (win == NULL)
+    return;
+
+  xid = hd_wm_watched_window_get_x_win (win);
+
+  gdk_error_trap_push ();
+
+  status = XGetWindowProperty(GDK_DISPLAY (), xid,
+                              hdwm->priv->atoms[HD_ATOM_NET_WM_STATE],
+                              0L, 1000000L,
+                              0, XA_ATOM, &realType, &format,
+                              &n, &extra, &value.cpp);
+
+  if (gdk_error_trap_pop ())
+  {
+    g_debug ("Some watched window view closed can be the cause");
+    return;
+  }
+
+  if (status == Success)
+  {
+    if (realType == XA_ATOM && format == 32 && n > 0)
+    {
+      for (i=0; i < n; i++)
+        if (value.a[i] && value.a[i] == hdwm->priv->atoms[HD_ATOM_NET_WM_STATE_FULLSCREEN])
+        {
+          if (value.a) XFree(value.a);
+          g_signal_emit_by_name (hdwm,"fullscreen",TRUE);
+	  g_debug ("FULLSCREEN ON %d",(GdkNativeWindow)xid);
+	  return;
+        }
+    }
+  }
+
+  if (value.a)
+    XFree(value.a);	      
+
+  g_signal_emit_by_name (hdwm,"fullscreen",FALSE);
+  g_debug ("FULLSCREEN OFF");
+
+}
+
 /* Main event filter  */
 
 static GdkFilterReturn
@@ -2273,6 +2352,9 @@ hd_wm_x_event_filter (GdkXEvent *xevent,
     if (!win)
       return GDK_FILTER_CONTINUE;
 
+    if (prop->atom == hdwm->priv->atoms[HD_ATOM_NET_WM_STATE])
+      hd_wm_check_net_state (hdwm, win);
+    else
     if (prop->atom == hdwm->priv->atoms[HD_ATOM_WM_NAME] || 
         prop->atom == hdwm->priv->atoms[HD_ATOM_MB_WIN_SUB_NAME] || 
 	prop->atom == hdwm->priv->atoms[HD_ATOM_NET_WM_NAME])
@@ -2724,6 +2806,8 @@ hd_wm_get_lowmem_timeout_multiplier(void)
 HDWMWatchedWindow *
 hd_wm_get_active_window(void)
 {
+if (hdwmpriv->active_window)
+    hd_wm_check_net_state (NULL,hdwmpriv->active_window);
   return hdwmpriv->active_window;
 }
 
@@ -2739,6 +2823,10 @@ hd_wm_reset_active_window(void)
 HDWMWatchedWindow *
 hd_wm_get_last_active_window(void)
 {
+
+  if (hdwmpriv->last_active_window)
+    hd_wm_check_net_state (NULL,hdwmpriv->last_active_window);
+
   return hdwmpriv->last_active_window;
 }
 
