@@ -38,8 +38,13 @@
 #include <hildon-widgets/hildon-file-chooser-dialog.h>
 #endif
 
+#ifdef HAVE_LIBHILDON
+#include <hildon/hildon-color-button.h>
+#include <hildon/hildon-caption.h>
+#else
 #include <hildon-widgets/hildon-color-button.h>
 #include <hildon-widgets/hildon-caption.h>
+#endif
 
 #include <string.h>
 
@@ -54,10 +59,6 @@
 #include <gtk/gtkcellrenderertext.h>
 #include <gtk/gtkcelllayout.h>
 
-#include <hildon-widgets/hildon-file-chooser-dialog.h>
-#include <hildon-widgets/hildon-color-button.h>
-#include <hildon-widgets/hildon-caption.h>
-
 #include "hd-home-background-dialog.h"
 #include "hd-home-l10n.h"
 #include "hd-home-background.h"
@@ -112,9 +113,7 @@ struct _HDHomeBackgroundDialogPrivate
   GtkWidget        *color_button;
   GtkWidget        *img_combo;
   GtkWidget        *mode_combo;
-#ifdef HAVE_LIBOSSOHELP
-  osso_context_t   *osso_context;
-#endif
+  gpointer          osso_context;
   
   HDHomeBackground *background;
 };
@@ -610,19 +609,32 @@ static void
 hd_home_background_dialog_color_changed (HDHomeBackgroundDialog *dialog)
 {
   HDHomeBackgroundDialogPrivate    *priv;
+#ifdef HAVE_LIBHILDON
+  GdkColor                          color;
+#else
   GdkColor                         *color;
+#endif
 
   priv = HD_HOME_BACKGROUND_DIALOG_GET_PRIVATE (dialog);
 
   if (!priv->background)
     return;
 
+#ifdef HAVE_LIBHILDON
+   hildon_color_button_get_color (HILDON_COLOR_BUTTON (priv->color_button),
+                                  &color);
+  
+   g_object_set (G_OBJECT (priv->background),
+                "color", &color,
+                NULL);
+#else
   color =
    hildon_color_button_get_color (HILDON_COLOR_BUTTON (priv->color_button));
 
   g_object_set (G_OBJECT (priv->background),
                 "color", color,
                 NULL);
+#endif
 }
 
 static void
