@@ -23,6 +23,10 @@
  *
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <errno.h>
 #include <sys/resource.h>
 #include <string.h>
@@ -40,7 +44,13 @@
 #include <X11/Xatom.h>
 
 #include <hildon-base-lib/hildon-base-dnotify.h>
+
+#ifdef HAVE_LIBHILDON
+#include <hildon/hildon-helper.h>
+#else
 #include <hildon-widgets/hildon-finger.h>
+#endif
+
 #include <libhildondesktop/libhildonmenu.h>
 #include <libhildondesktop/hildon-desktop-panel-window.h>
 #include <libhildondesktop/hildon-thumb-menu-item.h>
@@ -929,7 +939,11 @@ hn_others_button_button_release (GtkWidget *widget,
             	           button);
   }
 
+#ifdef HAVE_LIBHILDON
+  if (hildon_helper_event_button_is_finger (event))
+#else
   if (hildon_button_event_is_finger (event))
+#endif
     button->priv->thumb_pressed = TRUE;
 
   return TRUE;
